@@ -1,6 +1,6 @@
 from abc import ABC
 from argparse import Namespace, ArgumentParser
-from typing import List, Tuple, Optional, Dict, Callable
+from typing import List, Tuple, Optional, Dict, Union
 
 import numpy as np
 from numpy.random import RandomState
@@ -8,7 +8,7 @@ from numpy.random import RandomState
 from rlai.actions import Action
 from rlai.agents import Agent
 from rlai.states.mdp import MdpState
-from rlai.utils import sample_list_item, import_function
+from rlai.utils import sample_list_item
 
 
 class MdpAgent(Agent, ABC):
@@ -32,10 +32,15 @@ class MdpAgent(Agent, ABC):
 
         parser = ArgumentParser(allow_abbrev=False)
 
+        parser.add_argument(
+            '--continuous-state-discretization-resolution',
+            type=float,
+            help='Continuous-state discretization resolution.'
+        )
+
         parser.add_argument(
             '--gamma',
             type=float,
-            default=0.1,
             help='Discount factor.'
         )
 
@@ -43,47 +48,6 @@ class MdpAgent(Agent, ABC):
 
         return parsed_args, unparsed_args
 
-    @staticmethod
-    def parse_mdp_solver_args(
-            args
-    ) -> Tuple[Namespace, List[str]]:
-        """
-        Parse arguments for the specified MDP solver.
-
-        :param args: Aguments.
-        :return: 2-tuple of parsed and unparsed arguments.
-        """
-
-        parser = ArgumentParser(allow_abbrev=False)
-
-        parser.add_argument(
-            '--mdp-solver',
-            type=str,
-            help='Fully-qualified name of the MDP solver function to use.'
-        )
-
-        parser.add_argument(
-            '--theta',
-            type=float,
-            help='Minimum tolerated change in value estimates, below which policy evaluation terminates.'
-        )
-
-        parser.add_argument(
-            '--update-in-place',
-            action='store_true',
-            help='Update the policy in place (usually quicker).'
-        )
-
-        parser.add_argument(
-            '--evaluation-iterations-per-improvement',
-            type=int,
-            help='Number of policy evaluation iterations to execute for each iteration of improvement.'
-        )
-
-        parsed_args, unparsed_args = parser.parse_known_args(args)
-
-        return parsed_args, unparsed_args
-
     def initialize_equiprobable_policy(
             self,
             SS: List[MdpState]
@@ -105,23 +69,45 @@ class MdpAgent(Agent, ABC):
             for s in SS
         }
 
-    def solve_mdp(
-            self
-    ):
+    def get_state_i(
+            self,
+            state_descriptor: Union[str, np.ndarray]
+    ) -> int:
         """
-        Solve the current agent's MDP using the function and arguments passed to the constructor (e.g., from the
-        command line).
+        Get the integer identifier for a state. The returned value is guaranteed to be the same for the same state,
+        both throughout the life of the current agent as well as after the current agent has been pickled for later
+        use (e.g., in checkpoint-based resumption).
+
+        :param state_descriptor: State descriptor, either a string (for discrete states) or an array representing a
+        position within an n-dimensional continuous state space.
+
+        :return: Integer identifier.
         """
 
-        self.solver_function(self, **self.solver_function_args)
+        if isinstance(state_descriptor, np.ndarray):
+
+            if self.continuous_state_discretization_resolution is None:
+                raise ValueError('Attempted to discretize a continuous state without a resolution.')
+
+            state_descriptor = '|'.join(
+                str(int(state_dim_value / self.continuous_state_discretization_resolution))
+                for state_dim_value in state_descriptor
+            )
+
+        elif not isinstance(state_descriptor, str):
+            raise ValueError(f'Unknown state space type:  {type(state_descriptor)}')
+
+        if state_descriptor not in self.state_id_str_int:
+            self.state_id_str_int[state_descriptor] = len(self.state_id_str_int)
+
+        return self.state_id_str_int[state_descriptor]
 
     def __init__(
             self,
             name: str,
             random_state: RandomState,
-            gamma: float,
-            solver_function: Optional[Callable] = None,
-            solver_function_args: Optional[Dict] = None
+            continuous_state_discretization_resolution: Optional[float],
+            gamma: float
     ):
         """
         Initialize the agent with an empty policy. Call `initialize_equiprobable_policy` to initialize the policy for
@@ -129,9 +115,10 @@ class MdpAgent(Agent, ABC):
 
         :param name: Name of the agent.
         :param random_state: Random state.
+        :param continuous_state_discretization_resolution: A discretization resolution for continuous-state
+        environments. Providing this value allows the agent to be used with discrete-state methods via
+        discretization of the continuous-state dimensions.
         :param gamma: Discount.
-        :param solver_function: Solver function. Required in order to call `self.solve_mdp`.
-        :param solver_function_args: Solver function arguments. Required in order to call `self.solve_mdp`.
         """
 
         super().__init__(
@@ -139,11 +126,11 @@ class MdpAgent(Agent, ABC):
             random_state=random_state
         )
 
+        self.continuous_state_discretization_resolution = continuous_state_discretization_resolution
         self.gamma = gamma
-        self.solver_function = solver_function
-        self.solver_function_args = solver_function_args
 
         self.pi: Dict[MdpState, Dict[Action, float]] = {}
+        self.state_id_str_int: Dict[str, int] = {}
 
 
 class StochasticMdpAgent(MdpAgent):
@@ -165,24 +152,13 @@ class StochasticMdpAgent(MdpAgent):
         :return: 2-tuple of a list of agents and a list of unparsed arguments.
         """
 
-        # get the mdp solver and arguments
-        parsed_mdp_args, unparsed_args = cls.parse_mdp_solver_args(args)
-        solver_function = import_function(parsed_mdp_args.mdp_solver)
-        solver_function_arguments = {
-            k: v
-            for k, v in vars(parsed_mdp_args).items()
-            if k != 'mdp_solver' and v is not None
-        }
-
-        parsed_args, unparsed_args = cls.parse_arguments(unparsed_args)
+        parsed_args, unparsed_args = cls.parse_arguments(args)
 
         agents = [
             StochasticMdpAgent(
                 name=f'stochastic (gamma={parsed_args.gamma})',
                 random_state=random_state,
-                gamma=parsed_args.gamma,
-                solver_function=solver_function,
-                solver_function_args=solver_function_arguments
+                **vars(parsed_args)
             )
         ]
 
@@ -238,26 +214,25 @@ class StochasticMdpAgent(MdpAgent):
             self,
             name: str,
             random_state: RandomState,
-            gamma: float,
-            solver_function: Optional[Callable] = None,
-            solver_function_args: Optional[Dict] = None
+            continuous_state_discretization_resolution: Optional[float],
+            gamma: float
     ):
         """
         Initialize the agent.
 
         :param name: Name of the agent.
         :param random_state: Random state.
+        :param continuous_state_discretization_resolution: A discretization resolution for continuous-state
+        environments. Providing this value allows the agent to be used with discrete-state methods via
+        discretization of the continuous-state dimensions.
         :param gamma: Discount.
-        :param solver_function: Solver function. Required in order to call `self.solve_mdp`.
-        :param solver_function_args: Solver function arguments. Required in order to call `self.solve_mdp`.
         """
 
         super().__init__(
             name=name,
             random_state=random_state,
-            gamma=gamma,
-            solver_function=solver_function,
-            solver_function_args=solver_function_args
+            continuous_state_discretization_resolution=continuous_state_discretization_resolution,
+            gamma=gamma
         )
 
 
@@ -316,5 +291,6 @@ class Human(MdpAgent):
         super().__init__(
             name='human',
             random_state=None,
+            continuous_state_discretization_resolution=None,
             gamma=1
         )
