@@ -153,14 +153,18 @@ class KArmedBandit(Environment):
         return bandit, unparsed_args
 
     def reset_for_new_run(
-            self
+            self,
+            agent: Agent
     ) -> State:
         """
         Reset the the bandit, initializing arms to new expected values.
 
+        :param agent: Agent.
         :return: New State.
         """
 
+        super().reset_for_new_run(agent)
+
         # get new arm reward means and initialize new arms
         q_star_means = self.random_state.normal(loc=self.q_star_mean, scale=self.q_star_variance, size=self.k)
 
@@ -207,7 +211,7 @@ class KArmedBandit(Environment):
         """
 
         if self.random_state.random_sample() < self.reset_probability:
-            self.reset_for_new_run()
+            self.reset_for_new_run(agent)
 
         action = agent.act(t=t)
         monitor.report(t=t, agent_action=action, optimal_action=Action(self.best_arm.i))
@@ -222,6 +226,7 @@ class KArmedBandit(Environment):
     def __init__(
             self,
             random_state: RandomState,
+            T: Optional[int],
             k: int,
             q_star_mean: float,
             q_star_variance: float,
@@ -231,17 +236,19 @@ class KArmedBandit(Environment):
         """
         Initialize the bandit.
 
+        :param random_state: Random state.
+        :param T: Maximum number of steps to run, or None for no limit.
         :param k: Number of arms.
         :param q_star_mean: Mean of q_star.
         :param q_star_variance: Variance of q_star.
         :param reward_variance: Reward variance.
         :param reset_probability: Per-step probability of resetting (nonstationarity).
-        :param random_state: Random state.
         """
 
         super().__init__(
             name=f'{k}-armed bandit',
-            random_state=random_state
+            random_state=random_state,
+            T=T
         )
 
         self.k = k
