@@ -40,7 +40,7 @@ def evaluate_v_pi(
     for episode_i in range(num_episodes):
 
         # start the environment in a random state
-        state = environment.reset_for_new_run()
+        state = environment.reset_for_new_run(agent)
         agent.reset_for_new_run(state)
 
         # simulate until episode termination, keeping a trace of states and their immediate rewards, as well as the
@@ -48,7 +48,7 @@ def evaluate_v_pi(
         t = 0
         state_first_t = {}
         t_state_reward = []
-        while not state.terminal:
+        while not state.terminal and (environment.T is None or t < environment.T):
 
             if state not in state_first_t:
                 state_first_t[state] = t
@@ -58,10 +58,12 @@ def evaluate_v_pi(
             else:
                 a = agent.act(t)
 
-            next_state, next_t, reward = state.advance(environment, t, a)
+            next_state, reward = state.advance(environment, t, a, agent)
             t_state_reward.append((t, state, reward))
             state = next_state
-            t = next_t
+            t += 1
+
+            agent.sense(state, t)
 
         # work backwards through the trace to calculate discounted returns. need to work backward in order for the value
         # of G at each time step t to be properly discounted.
@@ -134,8 +136,9 @@ def evaluate_q_pi(
     episodes_per_print = max(1, int(num_episodes * 0.05))
     for episode_i in range(num_episodes):
 
-        # reset the environment for the new run, and reset the agent accordingly.
-        state = environment.reset_for_new_run()
+        # reset the environment for the new run (always use the agent we're learning about, as state identifiers come
+        # from it), and reset the episode generate agent accordingly.
+        state = environment.reset_for_new_run(agent)
         episode_generation_agent.reset_for_new_run(state)
 
         # simulate until episode termination, keeping a trace of state-action pairs and their immediate rewards, as well
@@ -144,7 +147,7 @@ def evaluate_q_pi(
         state_action_first_t = None if update_upon_every_visit else {}
         t_state_action_reward = []
         total_reward = 0.0
-        while not state.terminal:
+        while not state.terminal and (environment.T is None or t < environment.T):
 
             evaluated_states.add(state)
 
@@ -159,11 +162,11 @@ def evaluate_q_pi(
             if state_action_first_t is not None and state_a not in state_action_first_t:
                 state_action_first_t[state_a] = t
 
-            next_state, next_t, next_reward = state.advance(environment, t, a)
+            next_state, next_reward = state.advance(environment, t, a, agent)
             t_state_action_reward.append((t, state_a, next_reward))
             total_reward += next_reward.r
             state = next_state
-            t = next_t
+            t += 1
 
             episode_generation_agent.sense(state, t)
 
